package storage_file

import (
	"cake_crm/internal/modules/storage"
	crm "cake_crm/proto"
	"context"
	"encoding/json"
	"errors"
	"os"
)

type Product struct {
	Id              int64                 `json:"id"`
	Article         string                `json:"article"`
	Name            string                `json:"name"`
	Uri             string                `json:"uri"`
	Images          []string              `json:"images"`
	Description     string                `json:"description"`
	Group           int64                 `json:"group"`
	Unit            string                `json:"unit"`
	Inventory       float64               `json:"inventory"`
	Variants        []*crm.Variant        `json:"variants"`
	Characteristics []*crm.Characteristic `json:"characteristics"`
	Category        int64                 `json:"category"`
}

var (
	ErrProductNotFound = errors.New("product not found")
)

type storageFile struct{}

func NewStorageFile() storage.IStorage {
	return &storageFile{}
}

func (s *storageFile) GetCatalog(_ context.Context) ([]*crm.Category, error) {
	data, err := os.ReadFile("resources/catalog.json")
	if err != nil {
		return nil, err
	}
	var res []*crm.Category
	if err := json.Unmarshal(data, &res); err != nil {
		return nil, err
	}
	return res, nil
}

func (s *storageFile) GetPositions(_ context.Context, id int64) ([]*crm.Product, error) {
	data, err := os.ReadFile("resources/products.json")
	if err != nil {
		return nil, err
	}
	var products []*crm.Product
	if err := json.Unmarshal(data, &products); err != nil {
		return nil, err
	}
	res := make([]*crm.Product, 0, len(products))
	for _, product := range products {
		if id == 0 || product.Category == id {
			res = append(res, product)
		}
	}
	return res, nil
}

func (s *storageFile) GetProduct(_ context.Context, id int64) (*crm.Product, error) {
	data, err := os.ReadFile("resources/products.json")
	if err != nil {
		return nil, err
	}
	var products []*Product
	if err := json.Unmarshal(data, &products); err != nil {
		return nil, err
	}
	var product *Product
	for _, p := range products {
		if p.Id == id {
			product = p
			break
		}
	}
	if product == nil {
		return nil, ErrProductNotFound
	}
	res := &crm.Product{
		Id:              product.Id,
		Article:         product.Article,
		Name:            product.Name,
		Uri:             product.Uri,
		Images:          product.Images,
		Description:     product.Description,
		GroupedProducts: nil,
		Unit:            product.Unit,
		Inventory:       product.Inventory,
		Variants:        product.Variants,
		Characteristics: product.Characteristics,
		Category:        product.Category,
	}
	for _, p := range products {
		if p.Group == product.Group {
			image := ""
			if len(product.Images) > 0 {
				image = p.Images[0]
			}
			res.GroupedProducts = append(res.GroupedProducts, &crm.GroupedProduct{
				Name:  p.Name,
				Uri:   p.Uri,
				Image: image,
			})
		}
	}
	return res, nil
}

func (s *storageFile) GetBreadcrumbs(_ context.Context, id int64) ([]*crm.Category, error) {
	data, err := os.ReadFile("resources/catalog.json")
	if err != nil {
		return nil, err
	}
	var categories []*crm.Category
	if err := json.Unmarshal(data, &categories); err != nil {
		return nil, err
	}
	return getBreadcrumbs(categories, id), nil
}

func getBreadcrumbs(categories []*crm.Category, id int64) []*crm.Category {
	for _, category := range categories {
		if category.Id == id {
			category.Children = nil
			return []*crm.Category{category}
		}
		breadcrumbs := getBreadcrumbs(category.Children, id)
		if len(breadcrumbs) != 0 {
			res := make([]*crm.Category, 0, len(breadcrumbs)+1)
			category.Children = nil
			res = append(res, category)
			res = append(res, breadcrumbs...)
			return res
		}
	}
	return nil
}
